# -.- coding: utf-8 -.-

# Copyright (c) 2011-2012 Collabora, Ltd.
#
# Gnome Clocks is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Gnome Clocks is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Gnome Clocks; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# Author: Seif Lotfy <seif.lotfy@collabora.co.uk>

__version__ = "0.1.2"

# Credit every contributor, in alphabetical order
AUTHORS = [
    "Alex Anthony",
    "Paolo Borelli",
    "Allan Day",
    "Piotr Drąg",
    "Emily Gonyer",
    "Maël Lavault",
    "Seif Lotfy",
    "William Jon McCann",
    "Eslam Mostafa",
    "Bastien Nocera",
    "Jakub Steiner"
]

COPYRIGHTS =\
"""2011 © Collabora Ltd.
   2012 © Collabora Ltd., Seif Lotfy
          Emily Gonyer, Eslam Mostafa, Paolo Borelli"""
